﻿using AutomobilePlantContracts.BindingModels;
using AutomobilePlantContracts.BusinessLogicContracts;
using AutomobilePlantContracts.SearchModels;
using AutomobilePlantContracts.StorageContracts;
using AutomobilePlantContracts.ViewModels;
using AutomobilePlantDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.Metadata;
using System.Text;
using System.Threading.Tasks;

namespace AutomobilePlantBusinessLogic.BusinessLogics
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;

        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }

        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            _logger.LogInformation("ReadElement. Shop Name:{0}, ID:{1}", model.Name, model.Id);

            var element = _shopStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement. element not found");
                return null;
            }

            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);

            return element;
        }

        public List<ShopViewModel>? ReadList(ShopSearchModel? model)
        {
            _logger.LogInformation("ReadList. Shop Name:{0}, ID:{1} ", model?.Name, model?.Id);

            var list = (model == null) ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);

            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }

            _logger.LogInformation("ReadList. Count:{Count}", list.Count);

            return list;
        }

        public bool SupplyCars(ShopSearchModel model, ICarModel car, int count)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (car == null)
            {
                throw new ArgumentNullException(nameof(car));
            }

            if (count <= 0)
            {
                throw new ArgumentNullException("Count of cars in supply must be more than 0", nameof(count));
            }

            var shopElement = _shopStorage.GetElement(model);
            if (shopElement == null)
            {
                _logger.LogWarning("Required shop element not found in storage");
                return false;
            }

            _logger.LogInformation("Shop element found. ID: {0}, Name: {1}", shopElement.Id, shopElement.Name);

            var countCars = 0;
            foreach (var c in shopElement.ShopCars)
            {
                countCars += c.Value.Item2;
            }
            if (shopElement.MaxCountCars - countCars >= count)
            {
                if (shopElement.ShopCars.TryGetValue(car.Id, out var sameCar))
                {
                    shopElement.ShopCars[car.Id] = (car, sameCar.Item2 + count);
                    _logger.LogInformation("Same car found by supply. Added {0} of {1} in {2} shop", count, car.CarName, shopElement.Name);
                }
                else
                {
                    shopElement.ShopCars[car.Id] = (car, count);
                    _logger.LogInformation("New car added by supply. Added {0} of {1} in {2} shop", count, car.CarName, shopElement.Name);
                }
                _shopStorage.Update(new()
                {
                    Id = shopElement.Id,
                    Name = shopElement.Name,
                    Address = shopElement.Address,
                    OpeningDate = shopElement.OpeningDate,
                    ShopCars = shopElement.ShopCars,
                    MaxCountCars = shopElement.MaxCountCars
                });
            }
            else
            {
                _logger.LogWarning("Required shop is overflowed");
                return false;
            }

            return true;
        }

        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);

            if (_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }

            return true;
        }

        public bool SellCar(ICarModel car, int count)
        {
            return _shopStorage.SellCar(car, count);
        }

        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);

            if (_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }

            return true;
        }

        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);

            if (_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }

            return true;
        }

        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }

            if (!withParams)
            {
                return;
            }

            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет названия магазина!", nameof(model.Name));
            }

            if (model.MaxCountCars < 0)
            {
                throw new InvalidOperationException("Магазин с отрицательным количеством максимального количества авто!");
            }

            _logger.LogInformation("Shop. Name: {0}, Address: {1}, ID: {2}", model.Name, model.Address, model.Id);

            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                Name = model.Name
            });

            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }
    }
}
